/*
*	crob: cleaning robot
*	usage: crob -time dir ...
*	crob sweep out contents in the given dirs.
*	unit of time is 's','m','h','d'
*	names that is older than the given time are removed
*
*	ver. 1.0
*
*	example:
*	- remove files or dirs that are modified before 30 days from /tmp
*		crob -30d /tmp
*	- daily remove grey data in "/mail/grey/*" passed a day.
*	add following line to /cron/upas/cron
*		0 2 * * * HOST /usr/local/bin/386/crob -1d /mail/grey/*
*	where you should replace HOST by your cpu server name.
*/

#include <u.h>
#include <libc.h>
#include <ctype.h>
void mkclean(long t, char *dirname);
void err(char *f);
void rmdir(char *f);

void
usage(void)
{
	fprint(2,"usage: crob -time dir\n");
	fprint(2,"crob sweeps out old names in the subtree of given directory.\n");
	fprint(2,"unit of time is 's','m','h','d'\n");
	exits("usage");
}

void
main(int argc, char *argv[])
{
	char *p, *q;
	int u;
	long t, t0;
	argc--; argv++;
	if(argc < 2)
		usage();
	q= argv[0];
	if(*q != '-')
		usage();
	q++;
	p = q;
	while(isdigit(*p))
		p++;
	if(p == q)
		usage();
	u = 0;
	if(*p){
		switch(*p){
		case 's': u = 1; break;
		case 'm': u = 60; break;
		case 'h': u = 60*60; break;
		case 'd': u = 60*60*24; break;
		default: usage();
		}
	}
	else
		usage();
	t0 = time(nil);
	*p = 0;
	t = t0 - u * atoi(q);
	argv++;
	while(*argv)
		mkclean(t, *argv++);
}

void
mkclean(long t, char *dirname)
{
	int fd, i, n;
	Dir *dirbuf;

	fd = open(dirname, OREAD);
	if(fd < 0){
		err(dirname);
		return;
	}
	n = dirreadall(fd, &dirbuf);
	close(fd);
	if(n < 0){
		err("dirreadall");
		return;
	}
	chdir(dirname);
	for(i = 0; i < n; i++){
		if(dirbuf[i].mtime < t){
			if(dirbuf[i].mode & DMDIR)
				rmdir(dirbuf[i].name);
			else
				remove(dirbuf[i].name);
		}
	}
	free(dirbuf);
}


/*	stolen from rm.c	-Kenar- */

char	errbuf[ERRMAX];
int	ignerr = 0;

void
err(char *f)
{
	if(!ignerr){
		errbuf[0] = '\0';
		errstr(errbuf, sizeof errbuf);
		fprint(2, "rm: %s: %s\n", f, errbuf);
	}
}

/*
 * f is a non-empty directory. Remove its contents and then it.
 */
void
rmdir(char *f)
{
	char *name;
	int fd, i, j, n, ndir, nname;
	Dir *dirbuf;

	fd = open(f, OREAD);
	if(fd < 0){
		err(f);
		return;
	}
	n = dirreadall(fd, &dirbuf);
	close(fd);
	if(n < 0){
		err("dirreadall");
		return;
	}

	nname = strlen(f)+1+STATMAX+1;	/* plenty! */
	name = malloc(nname);
	if(name == 0){
		err("memory allocation");
		return;
	}

	ndir = 0;
	for(i=0; i<n; i++){
		snprint(name, nname, "%s/%s", f, dirbuf[i].name);
		if(remove(name) != -1)
			dirbuf[i].qid.type = QTFILE;	/* so we won't recurse */
		else{
			if(dirbuf[i].qid.type & QTDIR)
				ndir++;
			else
				err(name);
		}
	}
	if(ndir)
		for(j=0; j<n; j++)
			if(dirbuf[j].qid.type & QTDIR){
				snprint(name, nname, "%s/%s", f, dirbuf[j].name);
				rmdir(name);
			}
	if(remove(f) == -1)
		err(f);
	free(name);
	free(dirbuf);
}
